# bemmod.py
"""
	Generating coefficient matrix for acoustic scattering problems
	by BEM++. 
"""

import bempp.api
import scipy.io as sio

def bemmat(gname, k, d=0, smat='tmp.mat', ptype="DP"):
    grid = bempp.api.import_grid('data/msh/'+gname+'.msh')
    # grid.plot()
    
    # SET HMAT TRUNCATION THRESHOLD (default 1E-3), disable coarsening 
    bempp.api.global_parameters.hmat.eps = 1E-6
    # bempp.api.global_parameters.hmat.coarsening = False 

    # DEFINE FUNCTION SPACE
    piecewise_quadratic_space = bempp.api.function_space(grid, ptype, d)

    # DEFINE EACH MATRICES
    identity = bempp.api.operators.boundary.sparse.identity(
                piecewise_quadratic_space, piecewise_quadratic_space,
                piecewise_quadratic_space)
    adlp = bempp.api.operators.boundary.helmholtz.adjoint_double_layer(
                piecewise_quadratic_space, piecewise_quadratic_space,
                piecewise_quadratic_space,k)
    slp = bempp.api.operators.boundary.helmholtz.single_layer(
                piecewise_quadratic_space, piecewise_quadratic_space,
                piecewise_quadratic_space,k)

    lhs = .5*identity + adlp - 1j * k * slp

    # GENERATE AND SAVE MATRICES A
    lhs_discrete = lhs.weak_form()
    lhs_identity = identity.weak_form()
#    print("Shape of the matrix: {0}".format(lhs_discrete.shape))
    spmat = lhs_identity.sparse_operator;
    sio.savemat(smat,{'M':spmat})

    return lhs_discrete
