%   Convergence speed test of subspace method:
%       shifted Grcar matrix
%
%   NOTE: use MultiPrecision precision toolbox.
%
%   Created: 08-20-2016 by Ding.Lu@UNIGE.ch
%

close all;
pathdef_mptool; % define path to MultiPrecision toolbox

nd = 620;       % number of decimal digits in the experiment
nd_e = 700;     % decimal digits to produce 'exact' solution

N = 120;        % Data matrix: grcar matrix
x0 = -4000; 
y0 = 4000;
A0 = exp(pi*1/3*1i)*full(gallery('grcar',N)) - (4 + 2 * 1i) * eye(N); 


figure;         % PLOT: numerical range of A
numrange(A0); hold on; 
plot(0,0, '+r', 'markersize', 8, 'linewidth', 2);
axis([-7, 1, -5, 3]);
xlabel('real');
ylabel('imag');

%------------------------------ 
% Compute 'exact crawfno'
%------------------------------ 
mp.Digits(nd_e);
tol = mp.eps*1000;
AA = mp(A0);
mp_opts.mpi = mp('pi'); mp_opts.mpeps = mp.eps;
[cn0] = crawfnos_sg(AA, tol, 0, 2, false, 0, 'c', mp_opts);

%------------------------------ 
% Subspace alg, and 3-vector variant
%------------------------------ 

mp.Digits(nd);
tol = mp.eps*1000;
AA = mp(A0);

mp_opts.mpi = mp('pi'); mp_opts.mpeps = mp.eps;


fprintf('\n-------------Subspace CRAWFNO-----------\n')
rng('default'); % default random generator
[cn, z, info] = crawfnos_sg(AA, tol, 1, 1,  false, 0, 'c', mp_opts);
fprintf('-- CRAWFNO (subspace) = %15.14E, itrs = %3i \n', cn, info.its);

fprintf('\n-------------3v CRAWFNO-----------\n')
rng('default'); % default random generator
[cn3, z3, info3] = crawfnos_sg(AA, tol, 1, 2, false, 0, 'c', mp_opts);
fprintf('-- CRAWFNO (subspace) = %15.14E, itrs = %3i \n', cn3, info3.its);

p = info.PHI;
p3 = info3.PHI;
p00 = cn0; 

save ex1b.mat;

figure;
p = p(1:end-1);
plot(-log10(abs(p(1:end-1) - p00)), -log10(abs(p(2:end) - p00)),...
        'x-', 'linewidth', 2, 'markersize', 8);
hold on; plot(-[log10(tol), 0], -2.4142*[log10(tol), 0], '--');
legend('Full subspace', 'Slope 2.4142', ...
        'location', 'SouthEast', 'Interpreter', 'latex');
xlabel('$-\ln(e_{k-1})$','Interpreter', 'latex'); 
ylabel('$-\ln(e_{k})$','Interpreter', 'latex' );
axis([0,500, 0, 620]);

figure;
p3 = p3(1:end-1);
plot(-log10(abs(p3(1:end-1) - p00)), -log10(abs(p3(2:end) - p00)),...
        'x-r', 'linewidth', 2, 'markersize', 8);
hold on; plot(-[log10(tol), 0], -2.26953*[log10(tol), 0], '--r');
legend( '3-vector version', 'Slope 2.2695', ...
        'location', 'SouthEast', 'Interpreter', 'latex');
xlabel('$-\ln(e_{k-1})$','Interpreter', 'latex'); 
ylabel('$-\ln(e_{k})$','Interpreter', 'latex' );
axis([0,500, 0, 620]);

return
