%   Convergence speed test of subspace method:
%       flat portion on the boundary
%
%   NOTE: use MultiPrecision precision toolbox.
%
%   Created: 08-20-2016 by Ding.Lu@UNIGE.ch
%

close all;
pathdef_mptool; % define path to MultiPrecision toolbox

nd = 620;       % number of decimal digits in the experiment
nd_e = 700;     % decimal digits to produce 'exact' solution

N = 120;        % Data matrix: mixed diagonal with multiple eig with skewsym
A = diag([1, 1, 2 + 1/N*[3:1:N] ]); % multiplicity 2

% TST1
 B = -0*diag(ones(N,1)) + diag(ones(N-1,1),1) + diag(ones(N-1,1),-1);
 A0 = (A + B*1i);
 x0 = 0; y0 = 0.5;
 A0 = A0 - (x0 + y0 * 1i) * eye(N); 

% TST2: rand skew part
%rng('default'); 
%B = tril(randn(N,N),-1); B = B/norm(B); B = B + B';
%A0 = (A + B*1i);
%x0 = 0; y0 = 0.1;
%A0 = A0 - (x0 + y0 * 1i) * eye(N); 

% TST3: flat intersets nonflat vphi fun, (CHALLENGING EXAMPLE)
% WARNING: inner loop is extremely slow... Relate to Grcar matrix, almost flat
% surface..
%nd = 300;       % number of decimal digits in the experiment
%nd_e = 350;     % decimal digits to produce 'exact' solution
%
%N = 20;        
%k = 1; A = diag([ones(1,k), linspace(2,10, N-k)] + 1);
%x0 = 0; y0 = 1;
%A = A - (x0 + y0 * 1i) * eye(N);
%rng('default'); 
%B = tril(randn(N,N),-1); B = B/norm(B); B = B + B';
%A = (A + B*1i);
%
%AA = diag([ones(1,1), 1+.5*linspace(2,10, N-1)] + 1);
%BB = tril(randn(N,N),-1)*1i; BB = BB/norm(BB); BB = BB + BB';
%AA = AA + BB*1i;
%% Flat optimal
%A0 = blkdiag(AA, A); 
%

figure;         % PLOT: numerical range of A
numrange(A0); hold on; 
plot(0,0, '+r', 'markersize', 8, 'linewidth', 2);
xlabel('real');
ylabel('imag');
axis([-1,16,-3,3]);

%------------------------------ 
% Compute 'exact crawfno'
%------------------------------ 
mp.Digits(nd_e);
tol = mp.eps*1000;
AA = mp(A0);
mp_opts.mpi =  mp('pi');
mp_opts.mpeps =  mp('eps');
theta0 = mp('pi')/3;
[ra0] = crawfnos_sg(AA, tol, 1, 2, false, theta0, 'c',mp_opts);


%------------------------------ 
% Subspace alg, and 3-vector variant
%------------------------------ 

mp.Digits(nd);
tol = mp.eps*1000;
AA = mp(A0);
theta0 = mp('pi')/3;

mp_opts.mpi =  mp('pi');
mp_opts.mpeps =  mp('eps');

fprintf('\n-------------Subspace CRAWFNO-----------\n')
rng('default'); % default random generator
[ra, z, info] = crawfnos_sg(AA, tol, 1, 1, false, theta0, 'c', mp_opts);
fprintf('-- CRAWFNO (subspace) = %15.14E, itrs = %3i \n', abs(ra), info.its);

fprintf('\n-------------3v CRAWFNO-----------\n')
rng('default'); % default random generator
[ra3, z3, info3] = crawfnos_sg(AA, tol, 1, 2, false, theta0, 'c', mp_opts);
fprintf('-- CRAWFNO (3 vector version) = %15.14E, itrs = %3i \n', abs(ra3), info3.its);

p = info.PHI;
p3 = info3.PHI;
p00 = abs(ra0);
save ex1c.mat;


figure;
p = p(1:end);
plot(-log10(abs(p(1:end-1) - p00)), -log10(abs(p(2:end) - p00)),...
        'xk', 'linewidth', 2, 'markersize', 8);
hold on; plot(-[log10(tol), 0], -3*[log10(tol), 0], '--');
plot(-[log10(tol), 0], -1*[log10(tol), 0], '--r');
plot(-[log10(tol), 0], -2*[log10(tol), 0], '-b');
plot(-[log10(tol), 0], -1.5*[log10(tol), 0], '-r');
legend('Full subspace', 'Slope 3', ...
    'Slope 1', 'Ratio 2', 'Ratio 1.5', 'location', 'SouthEast');
xlabel('$-\ln(e_{k-1})$','Interpreter', 'latex'); 
ylabel('$-\ln(e_{k})$','Interpreter', 'latex' );
axis([0,500, 0, 620]);

fprintf('Full subspace ratio=\n');
x = -log10(abs(p(1:end-1) - p00));
y = -log10(abs(p(2:end) - p00));
r = (y(2:end) - y(1:end-1)) ./ (x(2:end) - x(1:end-1));
fprintf('   %16.15E \n', r);


figure;
p3 = p3(1:end);
plot(-log10(abs(p3(1:end-1) - p00)), -log10(abs(p3(2:end) - p00)),...
        'xk', 'linewidth', 2, 'markersize', 8);
hold on; plot(-[log10(tol), 0], -2.26953*[log10(tol), 0], '--');
plot(-[log10(tol), 0], -1*[log10(tol), 0], '--r');
legend( '3-vector version', 'Slope 2.2695', 'Slope 1.0', ...
        'location', 'SouthEast');
xlabel('$-\ln(e_{k-1})$','Interpreter', 'latex'); 
ylabel('$-\ln(e_{k})$','Interpreter', 'latex' );
axis([0,500, 0, 620]);

fprintf('3 vector subspace ratio=\n');
x = -log10(abs(p3(1:end-1) - p00));
y = -log10(abs(p3(2:end) - p00));
r = (y(2:end) - y(1:end-1)) ./ (x(2:end) - x(1:end-1));
fprintf('   %16.15E \n', r);





return


figure;
p = p(1:end);
plot(-log10(abs(p(1:end-1) - p00)), -log10(abs(p(2:end) - p00)),...
        'x-', 'linewidth', 2, 'markersize', 8);
hold on; plot(-[log10(tol), 0], -2.4142*[log10(tol), 0], '--');
plot(-[log10(tol), 0], -2*[log10(tol), 0], '-.g');
legend('Full subspace', 'Slope 2.4142', ...
        'location', 'SouthEast');
xlabel('$-\ln(e_{k-1})$','Interpreter', 'latex'); 
ylabel('$-\ln(e_{k})$','Interpreter', 'latex' );
axis([0,500, 0, 620]);

figure;
p3 = p3(1:end);
plot(-log10(abs(p3(1:end-1) - p00)), -log10(abs(p3(2:end) - p00)),...
        'x-r', 'linewidth', 2, 'markersize', 8);
hold on; plot(-[log10(tol), 0], -2.26953*[log10(tol), 0], '--r');
plot(-[log10(tol), 0], -2*[log10(tol), 0], '-.g');
legend( '3-vector version', 'Slope 2.2695', 'Slope 2', ...
        'location', 'SouthEast');
xlabel('$-\ln(e_{k-1})$','Interpreter', 'latex'); 
ylabel('$-\ln(e_{k})$','Interpreter', 'latex' );
axis([0,500, 0, 620]);
return
