%   Performance comparison with Uhlig's algorithm (crawfno)
%
%   Example
%       Grcar: Uhlig's alg converges slowly.
close all; clear all;

doplot = true;
N = 120;
A = full(gallery('grcar',N))*exp(pi*0/3*1i);
tol = 1.0E-13; 

xx = linspace(-20, 20, 30);
yy = linspace(-20, 20, 30);
xx = xx(xx>-11); xx = xx(xx<4);
yy = yy(yy>-6); yy = yy(yy<6);
lx = length(xx);
ly = length(yy);

if doplot % Plot the numerical range.
    figure(1);
    [X,Y] = meshgrid(xx, yy);
    plot(X,Y,'.k');
    hold on;
    numrange(A); 
    axis auto;
end

CN = zeros(ly, lx);
CN0 = zeros(ly, lx);
CN1 = zeros(ly, lx);
CN2 = zeros(ly, lx);
RUN0 = zeros(ly, lx);
RUN1 = zeros(ly, lx);
RUN2 = zeros(ly, lx);

opts1.tol = tol; opts1.mode = 'b'; 
opts2.tol = tol; opts2.mode = 'r'; 
parfor i = 1:lx
    for j = 1:ly
        x0 = xx(i); 
        y0 = yy(j);
        AA = A - (x0 + y0 * 1i) * eye(N); 
        % run Uhlig's alg
        [ra0, ~, run0] = crawfno(AA, 0, 0, tol, 0, 1000);
        % run full subspace 
        [cn1, ~, info1 ]=crawfnos(AA, opts1);
        % run reduced subspace 
        [cn2, ~, info2 ]=crawfnos(AA, opts2);
        % "exact solution" by bisection
        [cn] = crawfno_bisect(AA, eps);

        if ra0 >0
        CN(j,i) = max([cn,0]); % only return the positive number
        CN0(j,i) = max([ra0,0]); % only return the positive number
        CN1(j,i) = cn1;
        CN2(j,i) = cn2;

        RUN0(j,i) = run0;
        RUN1(j,i) = info1.its;
        RUN2(j,i) = info2.its;
        end
    end
end

[IDX] = find(RUN0>30);
figure(1);
plot(X(IDX), Y(IDX), 'xr', 'markersize', 8, 'linewidth', 2);
text(X(IDX), Y(IDX)+.6, num2str(RUN0(IDX)));
xlabel('real')
ylabel('imag')
ylim([-6, 6]);
xlim([-11, 4]);


fprintf('Uhlig'' alg  : max its=%3i, mean its = %4.1f\n',...
        max(max(RUN0)), mean(mean(RUN0)));
fprintf('Subspace Full : max its=%3i, mean its = %4.1f, maxerr = %2.1E \n',...
        max(max(RUN1)), mean(mean(RUN1)), max(max(abs(CN-CN1))));
fprintf('Subspace 3v : max its=%3i, mean its = %4.1f, maxerr = %2.1E \n',...
        max(max(RUN2)), mean(mean(RUN2)), max(max(abs(CN-CN2))));

figure(2)
semilogy(sort(reshape(abs(CN0-CN)./abs(CN),1,[])),'xr', 'markersize', 8, 'linewidth', 2); hold on;
semilogy(sort(reshape(abs(CN1-CN)./abs(CN),1,[])),'+k', 'markersize', 8, 'linewidth', 2);
semilogy(sort(reshape(abs(CN2-CN)./abs(CN),1,[])),'<b', 'markersize', 8, 'linewidth', 2);
ylim([0, 1.0E-11]);
legend('Uhlig''s Alg.', 'Full subspace', '3-vector version', 'location',...
'northwest');
xlabel('experiments')
ylabel('relative error')
return;
