% Test for subspace method on the BEM problem
%
% NOTE: BEM++ software should be installed
%
% Created: 04-06-2017 by Ding.Lu@UNIGE.ch
%
% IMPORTANT: in matlnx10 run 
%   LD_PRELOAD="/usr/lib/libblas.so" matlab
% to start matlab.

clear all; format longe;
rng('default')

% python setting up.. add current folder
P = py.sys.path;
if count(P, '') == 0
    insert(py.sys.path, int32(0), '');
end


%MNAME = {'Lh02'; 'Lh01'; 'Lh005'; 'Lh002'};     % check folder /data/msh for available meshes
MNAME = {'Lh02'};     % check folder /data/msh for available meshes
K = {10};               % wave number
D = {1};               % 1 linear / 2 quadratic elements
ptype = 'DP';

for ii = 1:length(MNAME)

    mname = MNAME{ii};

    for jj = 1:length(K)

        k = K{jj};

        for kk = 1:length(D)

            d = D{kk};

            % Assembling linear operator by BEMpp, pass to MATLAB
            fprintf(['--- Mesh = ',mname,' wavenumber = ', num2str(k), ' degree = ', num2str(d), '\n']);
            fprintf('... generating linear operator by BEMpp ...\n')

            tmpmat = ['tmp_',mname,'.mat'];
            tic
            BemObj = py.bemmod.bemmat(mname,k,d,tmpmat,ptype);
            %   ** FIXED A BUG IN BEMPP: **
            %       bempp/api/assembly/discrete_boundary_operator.py
            %   scaled a*A adjoint be conj(a)*A', instead of a*A'.
            %

            SpM = load(tmpmat); delete(tmpmat);
            R = chol(SpM.M); % R'*R = SpM
            
            %   define MATLAB linear operator
            n = cell(BemObj.shape); n = double(n{1});

            %   matrix, conjugate matrix, vector multiplication
            %   matrix matrix multiplication;
            p2m = @(y) cell2mat( cell( y.tolist)).'; % python ndarray to vector
            m2p = @(y) py.numpy.array(real(y.')) + 1j*py.numpy.array(imag(y.')); % matlab (complex) vector to python

            Afun.n = n;
            Afun.matvec = @(x) R.'\(p2m( BemObj.matvec(m2p(R \ x)) ));    
            Afun.matmat = @(X) colfun(Afun.matvec, X);     
            BemObjH = BemObj.H;
            Afun.rmatvec = @(x) R.'\(p2m( BemObjH.matvec(m2p( R \ x)) ));   


            Mem =  BemObj.memory;
            t = toc; 
            fprintf('... finish assembling in %5.4f seconds\n... problem size n=%5i, meomory = %2.1f MB \n', t, n,Mem/2^10);

            % Timing estimates
            xx = randn(n, 1);
            tt = cputime;
            yy = Afun.rmatvec(xx);
            tt = cputime - tt;
            fprintf('... 1 MatVec time = %4.3fs,  1 eigs estimate (30its) = %4.3fs\n', tt*2, tt*2*15*30);

            % Compute by subspaces.
            opts1.dodisp = 1;
            tic;
            t1 = cputime; 
            fprintf('... full subspace methods \n')
            [c, dir, info] = crawfnos(Afun, opts1) ;
            t1 = cputime - t1;
            tt1 = toc;
            fprintf('... finish in %5.4f seconds \n', t1);

            % Compute by Uhlig's.
            tic;
            t2 = cputime;
            fprintf('... Uhlig alg \n')
            [c2, dir2, run2, hist2] = crawfno(Afun, 1, 0, 1.0E-13, eps);
            t2 = cputime - t2;
            fprintf('... finish in %5.4f seconds ...\n', t2);
            tt2 = toc;

            % Compute by subspaces. 3 vector version
            opts3.dodisp = 1;
            opts3.mode = 'r';
            tic;
            t3 = cputime; 
            fprintf('... 3v subspace methods \n')
            [c3, dir3, info3] = crawfnos(Afun, opts3) ;
            t3 = cputime - t3;
            tt3 = toc;
            fprintf('... finish in %5.4f seconds \n', t3);


            % save data
            prob.n = n; prob.d = d; prob.k = k; prob.Mem = Mem;
            subsp.t1 = t1; subsp.tt1 = tt1; subsp.c = c; subsp.dir = dir; subsp.info = info;
            uhlig.t2 = t2; uhlig.tt2 = tt2; uhlig.c2 = c2; uhlig.dir2 = dir2; uhlig.run2 = run2; uhlig.hist2 = hist2; 
            
            subsp3.t3 = t3; subsp3.tt3 = tt3; subsp3.c3 = c3; subsp3.dir3 = dir3; subsp3.info3 = info3;
            save([mname,'k',num2str(k),'d',num2str(d), ptype,'.mat'], 'prob', 'subsp', 'uhlig', 'subsp3'  );
        end
    end
end
