function [crawford, ra, run, p] = crawfno_moduhlig(A,pr,gr,errtol,kryltol,kryln, mpi)
% MODIFIED Uhlig's algorithm, to support multiprecision tool box.
%
% crawfno computes the generralized Crawford number for a square matrix A
% Input : A is a square matrix, real or complex
%         if pr = 1, intermediate Crawford data is displayed on-screen in each iteration step
%         if gr = 1, plot the FOV of A and computed intermediate points, directions and/or normals
%         kryltol specifies the break-off tolerance of the Kryov method eigen value finder
%             (default: 10^-10)
%         errtol defines the relative final break-off tolerance (default set to 10^-13)
%         kryln specifies the dimension of A above which we use Krylov eigenfinders,
%             rather than Francis' QR algorithm (default: size(A) = n = 200)

% Default settings etc:
if nargin == 1, pr = 0; gr = 0; errtol = 10^-13; kryltol = 10^-10; kryln = 200; end
if nargin == 2, gr = 0; errtol = 10^-13; kryltol = 10^-10; kryln = 200; end
if nargin == 3, errtol = 10^-13; kryltol = 10^-10; kryln = 200; end,
if nargin == 4, kryltol = 10^-10; kryln = 200; end
if nargin == 5, kryln = 200; end                             % preparations and settings :

    %
    angle_tol = min([10^-14, errtol*100]);
    tol_1 = min([10^-14, errtol*100]);
    tol_2 = min([10^-15, errtol*10]);

    if nargin == 7
        pi = mpi;
    end
    %

opts.disp = 0; opts.maxit = 1600; %opts.tol = kryltol; 

crawford = pi; bal = 0;
ra = [0;0]; U = [0 -1;1 0]; done = 0; kin = 0; kr = 0; [n m] = size(A);
if n ~= m, fprintf(' Matrix A is not square'), run = 0; return, end

    kryln = inf;

if gr == 1, wber3(A,40,opts,gr,kryln), end

% Start-up : (up to) four corners of an inner convex hull for FOV(A):
Ar = (A+A')/2; Ai = (A-A')/(2i); trA = trace(A)/n; trA = [real(trA);imag(trA)]; % a inside FOV(A)
ra = [1;0]; Atest = Ar; [PE PW] = eigval(A,Atest,n,opts,kryln,2);  % first direction and bdry pt pair
ran = [0;1]; Atest = Ai; [PN PS] = eigval(A,Atest,n,opts,kryln,2); % second FOV boundary pt pair
if n < kryln, run = 2; else run = 4; end                           % count eigen evaluations QR/Krylov
rara = [-ran,ra,ran,-ra]; P = [PN,PW,PS,PE;rara;U*rara];           % closed FOV bdry, counterclockwise
[junk, k] = min(abs(P(1,:) + 1i*P(2,:))); a = trA;
if junk <= eps, crawford = 0; ra = [0;0]; if gr == 1, hold off, end, return, end % zero on FOV bdy
PPPc = P(1,1:end) - P(1,[2:end,1]) + 1i*(P(2,1:end) - P(2,[2:end,1]));
jj = find(abs(PPPc) < angle_tol*max([abs(PPPc),eps])); jj = fliplr(jj); % find almost equal FOV bdry pts
if isempty(jj) == 0 && jj(1) == 4, P(:,4) = []; jj(1) = []; end    % wrap around if at end
for k = jj, if size(P,2) > 1, P(:,k) = []; end, end                % and eliminate those (except last)
leii = size(P,2);
if leii == 1, fprintf('A is a scalar multiple of I_n ! \n'),       % single point FOV
  ra = A(1,1); crawford = norm(ra); if gr == 1, hold off, end, return, end
if leii == 2                                                       % 2 distinct bdry pts: perpendicular
  ra = P(1:2,2) - P(1:2,1); ra = [-ra(2);ra(1)]; ra = ra/norm(ra); add = 0;
  Atest = ra(1)*Ar + ra(2)*Ai; [Pone,Ptwo] = eigval(A,Atest,n,opts,kryln,2);
  if n > kryln, run = run + 2; else run = run + 1; end
  if abs((Pone - P(1:2,1))'*ra) > angle_tol*norm(P(1:2,1))
    P = [P(:,1),[Pone;-ra;-U*ra],P(:,2:end)]; add = 1; leii = leii + 1; end
  if abs((Ptwo - P(1:2,1))'*ra) > angle_tol*norm(P(1:2,1))
    P = [P(:,1:end),[Ptwo;ra;U*ra]]; add = 1; leii = leii + 1; end,
  if add == 0
    [crawford ra] = strlineedge(P(1:2,1),P(1:2,2),1); 

    % ADDED BY DING 
    crawford = abs(crawford);
    % ======

    if gr == 1, hold off, end, return
  else P = [P,P(:,1)]; end,                                        % wrap points around
else P = [P,P(:,1)]; end
if gr == 1, plot(a(1),a(2),'ob'), plot(P(1,1),P(2,1),'*b'),        % plot starting inner polygon
  plot(P(1,2),P(2,2),'*r'), plot(P(1,3),P(2,3),'*g'),
  plot([P(1,1),P(1,1)+P(3,1)],[P(2,1),P(2,1)+P(4,1)],'-b'),
  plot([P(1,1),P(1,1)+P(5,1)],[P(2,1),P(2,1)+P(6,1)],'-m'),
  plot([P(1,2),P(1,2)+P(3,2)],[P(2,2),P(2,2)+P(4,2)],'-r'),
  plot([P(1,2),P(1,2)+P(5,2)],[P(2,2),P(2,2)+P(6,2)],'-m'),
  plot([P(1,3),P(1,3)+P(3,3)],[P(2,3),P(2,3)+P(4,3)],'-g'),
  plot([P(1,3),P(1,3)+P(5,3)],[P(2,3),P(2,3)+P(6,3)],'-m'),
  if leii > 3,
    plot(P(1,3),P(2,3),'*g'),
    plot([P(1,4),P(1,4)+P(3,4)],[P(2,4),P(2,4)+P(4,4)],'-y'),
    plot([P(1,4),P(1,4)+P(5,4)],[P(2,4),P(2,4)+P(6,4)],'-m'),
    plot(P(1,:),P(2,:),'-k'); end, end

% Intermediate updates and tightening of feasible FOV edge point angles :
a = sum(P(1:2,1:leii),2)/leii; if gr == 1, plot(a(1),a(2),'sb'), end % a inside inner convex hull
io = inout(P(1:2,:),a); inbreak = 0; coszeroangle = -1; cosold = -1;
if io == 1                % Case I1: zero lies outside the first inner convex F(A) approximation
  [junk,k] = min(abs(P(1,:)+1i*P(2,:))); [~, ~, d] = closestproj(P(1:2,:));
  if abs(P(5:6,k)'*(P(1:2,k))) < tol_1 && junk < min(abs(d)) % search dir is perpendicular to FOV pt
    crawford = junk; ra = P(1:2,k); ra = ra/junk;             % and closer to any edge projection
    if gr == 1, hold off; end, return, end
  innen = inangle(P,a); kout = 0;                 % find strip perp to an edge containing zero
  if isempty(innen)                               % not projecting onto FOV inner convex hull edge
    ra = P(1:2,k); ra = ra/norm(ra);
  else                                            % projecting onto FOV inner convex hull edge
    P1 = P(:,innen); P2 = P(:,innen+1);           % now find additonal FOV bdry pt:
    ra = P2(1:2) - P1(1:2); ra = [-ra(2);ra(1)]; ra = ra/norm(ra); end% project onto sector edge
  Atest = ra(1)*Ar + ra(2)*Ai; Pn = [eigval(A,Atest,n,opts,kryln);-ra;-U*ra]; run = run + 1;
  if isempty(innen) == 0
    if norm(Pn(1:2) - P1(1:2)) < tol_1*norm(P1(1:2)) || norm(Pn(1:2) - P2(1:2)) < tol_1*norm(P1(1:2))
      [crawford ra] = strlineedge(P1(1:2),P2(1:2),1);
      if gr == 1, hold off, end, return, end, end
  if gr == 1, plot(Pn(1),Pn(2),'*g'), end
  if abs(Pn(5:6)'*(Pn(1:2))) < tol_1             % direction nearly orthogonal to tangent:
    ra = Pn(1:2); crawford = norm(ra); ra = ra/crawford;
    if gr == 1, hold off; end, return, end
  [P k1] = insert(Pn,P);                          % insert new FOV bdry pt
  if k1 > 0, P1 = P(:,k1); P2 = P(:,k1+2);
  else
    if k == 1, kf = size(P,2) - 1; kn = 1; kt = 2;                % wrap around seam
    else kf = k - 1; kn = k; kt = k + 1; end
    P1 = P(:,kf); Pn = P(:,kn); P2 = P(:,kt); k1 = k; end         % choose P1, Pn, P2 triple
  if (norm(Pn(1:2) - P1(1:2)) < tol_2 || norm(Pn(1:2) - P2(1:2)) < tol_2) % corner of FOV found
    [crawford ra] = strlineedge(P1(1:2),P2(1:2),1);
    if gr == 1, hold off, end, return, end
  if gr == 1, plot([Pn(1),Pn(1)+Pn(3)],[Pn(2),Pn(2)+Pn(4)],'-k'), plot(Pn(1),Pn(2),'*k'), end
  while ((coszeroangle < -.8 && coszeroangle < cosold) || kout < 8) && coszeroangle < 0.9 &&...
      io && kout < 14                                       % refine bdry point list
    innen = inangle3([P1,Pn,P2]); cosold = coszeroangle; kout = kout + 1;
    if innen == 1, coszeroangle = P1(1:2)'*Pn(1:2)/(norm(P1(1:2))*norm(Pn(1:2)));
    else coszeroangle = P2(1:2)'*Pn(1:2)/(norm(P2(1:2))*norm(Pn(1:2))); end
    if innen == 1, ra = Pn(1:2) - P1(1:2); else ra = P2(1:2) - Pn(1:2); end  % new search direction
    if norm(ra) < tol_2                                          % FOV corner found again:
      [crawford ra] = strlineedge(P(1:2,1),P(1:2,2),1); end
    ra = [ra(2);-ra(1)]; ra = -ra/norm(ra);                       % refine bdry point list
    Atest = ra(1)*Ar + ra(2)*Ai; Pn = [eigval(A,Atest,n,opts,kryln);-ra;-U*ra];
    singvals = svd([Pn(1:2)-P1(1:2),P2(1:2)-P1(1:2)]); % measure lin independence of P1Pn, P1P2
    if singvals(2)/singvals(1) < tol_1 && kout > 7    % Pn on straight line P1 to P2 and FOV boundary
      [crawford ra] = strlineedge(P1(1:2),P2(1:2),1);
      if gr == 1, hold off, end, return, end
    if innen == 1, P2 = P(:,k1+1);
    else k1 = k1 + 1; P1 = P(:,k1); end                % sort bdry points counterclockwise
    P = [P(:,1:k1),Pn,P(:,k1+1:end)]; run = run + 1; nPnold = norm(Pn(1:2)); io = inout(P(1:2,:),a);
    if pr == 1,
      fprintf('outrun = %g , Pnnormal * Pn = %g , ||Pn|| = %5.15e \n',run,Pn(5:6)'*Pn(1:2),norm(Pn(1:2))),end
    if gr == 1,
      plot([Pn(1),Pn(1)+Pn(3)],[Pn(2),Pn(2)+Pn(4)],'-k'), plot(Pn(1),Pn(2),'+r'), end, end, end

if io == -1  Pn = PN;     % Case I2: zero lies inside first inner F(A) approximation
  PC = P(1,:) + 1i*P(2,:); minr = min(abs(PC)); maxr = max(abs(PC));
  if size(P,2) == 5 && maxr - minr < tol_2        % FOV edge is a circle or polygon of circle pts
    bal = 1;
    if norm(P(1:2,1) + P(1:2,3)) < tol_2 && norm(P(1:2,2) + P(1:2,4)) < tol_2 % antipodes
      for jj = 1:3                                 % check if more antipodes ==> disc as FOV
        wi = pi*rand(1); ra = [cos(wi);sin(wi)]; Atest = ra(1)*Ar + ra(2)*Ai; % random rotation
        [Pn Qn] = eigval(A,Atest,n,opts,kryln,2); if n < kryln, run = 2; else run = 4; end
        Pn = [Pn;-ra;ra(2);-ra(1)]; Qn = [Qn;ra;-ra(2);ra(1)];                % new FOV bdry pts
        if abs(Pn(1:2)'*[ra(2);-ra(1)]) + abs(Qn(1:2)'*[-ra(2);ra(1)]) > tol_2, bal = 0; end
        if gr == 1, plot(Pn(1),Pn(2),'+r'), plot(Qn(1),Qn(2),'+b'), end,
        P = insert(Pn,P); P = insert(Qn,P); end                               % insert into P
      if gr == 1, plot(P(1,:),P(2,:),'-m'), end, end   % F(A) is the FOV of one Jordan block J(k,0)
    PC = P(1,:) + 1i*P(2,:); minr = min(abs(PC)); maxr = max(abs(PC));
    if bal == 1 && maxr - minr < tol_2                  % circular FOV boundary around zero
      crawford = -minr; ra = [1;0]; if crawford == 0, ra = [0;0]; end
      if gr == 1, hold off, end, return, end, end
  if norm(A'*A - eye(n,n)) < angle_tol % A nearly unitary; FOV is convex hull of evs
    e = eig(A); run = run + 1;
    [~,k] = max(real(e)); [~,l] = min(real(e)); [~,ki] = max(imag(e)); [~,li] = min(imag(e));
    P = [real(e(k)) real(e(ki)) real(e(l)) real(e(li)) real(e(k));            % outer frame of FOV
      imag(e(k)) imag(e(ki)) imag(e(l)) imag(e(li)) imag(e(k))];
    e([k,l,ki,li]) = [];
    for ji = 1:size(e), P = insert([real(e(ji));imag(e(ji))],P); end
    [ra , ~, d] = closestproj(P); crawford = -min(d);
    if gr == 1, hold off, end, return, end
  
  while inbreak == 0                            % general case: we refine the bdry point list
    [~,k] = min(abs(P(1,:)+1i*P(2,:)));
    nPnold = norm(Pn(1:2)); [ra k1] = closestproj(P(1:2,:)); Atest = ra(1)*Ar + ra(2)*Ai;
    Pn = [eigval(A,Atest,n,opts,kryln);-ra;-U*ra];
    P1 = P(:,k1); P2 = P(:,k1+1); run = run + 1;
    nP1P2overPn = norm(P1(1:2)-P2(1:2))/norm(Pn(1:2));
    if (norm(Pn(1:2) - P1(1:2)) < tol_2 || norm(Pn(1:2) - P2(1:2)) < tol_2) % repeated bdry point :
      Ei = P2(1:2) - P1(1:2); ra = [Ei(2);-Ei(1)]/norm(Ei); crawford = -P1(1:2)'*ra;
      if crawford == 0, ra = [0;0]; end, if gr == 1, hold off, end, return, end
    singvals = svd([Pn(1:2)-P1(1:2),P2(1:2)-P1(1:2)]);        % measure lin independence of P1Pn, P1P2
    if singvals(2)/singvals(1) < tol_1,             % Pn on straight line P1 to P2 and boundary of FOV
      Ei = P2(1:2) - P1(1:2); ra = [Ei(2);-Ei(1)]/norm(Ei); crawford = -P1(1:2)'*ra;
      if gr == 1, hold off, end, return, end
    P = [P(:,1:k1),Pn,P(:,k1+1:end)]; kin = kin + 1;          % insert new bdry pt Pn into list
    [~, kn dn] = closestproj(P(1:2,:)); drest = dn; drest(kn) = [];
    B = P(3:4,kn:kn+1)'; b = diag(B*P(1:2,kn:kn+1)); x = B\b; maxdPk = norm(x); % farthest outside corner
    if gr == 1
      plot(Pn(1),Pn(2),'+r'), plot(P(1,k1:k1+2),P(2,k1:k1+2),'-b'), plot(x(1),x(2),'*b'), end
    if pr == 1
      fprintf(' inrun = %g , Pnnormal * Pn = %g , ||Pn|| = %5.15e \n',run,Pn(5:6)'*Pn(1:2),norm(Pn(1:2))), end
    if min(drest) > maxdPk || nP1P2overPn < .5      % minimal distance edge located for finishing
      k1 = min(size(P,2)-2,kn); P1 = P(:,k1); Pn = P(:,k1+1); P2 = P(:,k1+2); nPnold = norm(Pn(1:2));
      if gr == 1,
        plot([P1(1),P1(1)+P1(3)],[P1(2),P1(2)+P1(4)],'-k'),
        plot([P2(1),P2(1)+P2(3)],[P2(2),P2(2)+P2(4)],'-k'),
        plot([Pn(1),Pn(1)+Pn(3)],[Pn(2),Pn(2)+Pn(4)],'-k'), end, inbreak = 1; end, end, end

% Final iterations :
while abs(nPnold - norm(Pn(1:2)))/nPnold >= errtol || kr < 3
  in = inangle3([P1,Pn,P2]); nPnold = norm(Pn(1:2)); kr = kr + 1; % use circle interpolations now:
  if in == 1, P2 = Pn; else P1 = Pn; k1 = k1 + 1; end
  n1 = P1(5:6)'; n2 = P2(5:6)'; B = [n1;n2;0 0]; b = [n1*P1(1:2);n2*P2(1:2);0]; % find circle center and
  TT = B\b; raold = ra; ra = TT/norm(TT);                      %   use it as new direction
  if norm(raold - ra) < angle_tol, done = 1; end                  % do directions nearly coincide ?
  if done == 0                                                 % refine at least 5 times unless done = 1
    Atest = ra(1)*Ar + ra(2)*Ai; Pn = [eigval(A,Atest,n,opts,kryln);-ra;-U*ra];
    P = [P(:,1:k1),Pn,P(:,k1+1:end)]; run = run + 1;           % insert new pt
    if pr == 1
      fprintf('  frun = %g ,Pnnormal * Pn = %g , ||Pn|| = %5.15e \n',run,Pn(5:6)'*Pn(1:2),norm(Pn(1:2))),end
    if gr == 1,
      plot([Pn(1),Pn(1)+Pn(3)],[Pn(2),Pn(2)+Pn(4)],'-k'), plot(Pn(1),Pn(2),'.y'),
      plot(TT(1),TT(2),'ok'), end, end
  if abs(nPnold - norm(Pn(1:2)))/nPnold < errtol || done == 1  % check corner convergence
    innen = inangle3([P1,Pn,P2]);
    if innen == 1, p1 = P1(1:2); Ei = Pn(1:2) - p1; else p1 = Pn(1:2); Ei = P2(1:2) - p1; end
    al = -p1'*Ei/(Ei'*Ei);
    if (al >= 0 && al <= 1) && io == 1                         % edge pt is closest, or
      Ei = [-Ei(2);Ei(1)]; Ei = Ei/norm(Ei); crawford = Pn(1:2)'*Ei; ra = io*Ei;
    else  crawford = io*norm(Pn(1:2)); ra = io*ra; end         %   corner pt is closest
    if gr == 1, hold off, end, 
    
    p = [Pn(1) Pn(2)];

    return, end, end
p = V1'*A*V1 % touching points

%   ----------------------------------------------------------------   % END of main program

% Nine subroutine m-files:

function [P k] = insert(Pn,P)     % Insert new FOV bdry point Pn into P in counterclockwise order
E = P(1:2,2:end) - P(1:2,1:end-1); ON = [E(2,:);-E(1,:)]; k = -1;
for j = 1:size(ON,2),
  if ON(:,j)'*(Pn(1:2) - P(1:2,j)) > 0 && ON(:,j)'*(Pn(1:2) - P(1:2,j+1)) > 0, k = j; end, end
if k > 0, P = [P(:,1:k),Pn,P(:,k+1:end)]; end

function [cr ra] = strlineedge(P1,P2,io)       % Crawford data for straight line FOV edge
Ei = P2 - P1;
if Ei'*P1 < 0 && Ei'*P2 > 0                  % 0 projects onto line FOV edge
  ra = io*[-Ei(2);Ei(1)]/norm(Ei); cr = io*P1(1:2)'*ra;
else                                         % else choose closest bdry pt
  [junk,K] = min([norm(P1),norm(P2)]); cr = junk; ra = P1;
  if K == 2, ra = P2; end, end
if cr == 0, ra = [0;0]; end                  % check whether zero on FOV boundary

function [ra k d] = closestproj(P) % find closest projection point onto the edges between P points
E = P(:,2:end)-P(:,1:end-1); Ec = E(1,:)+1i*E(2,:); Eca = abs(Ec); % edges
En = [0 1;-1 0]*[E(1,:)./Eca; E(2,:)./Eca];                        % unit outward normals
d = diag(P'*En); [~,k] = min(d); ra = -En(:,k);

function io = inout(P,a)       % io = -1: zero lies in convex hull of P; io = 1: zero lies outside
En = edgenormals(P); C = (P(1:2,2:end)+P(1:2,1:end-1))/2;          % edge normals and centers
EnC = diag(En'*C); EnCmia = diag(En'*(C-a*ones(1,size(C,2))));     % directions
if all(EnC.*EnCmia >= 0), io = -1; else io = 1; end

function innen = inangle3(P)   % if innen = k ==> zero lies in kth angular wedge (|P| = 3)
innen = 2; if P(5:6,1)'*P(1:2,1) < 0 && P(5:6,2)'*P(1:2,2) > 0 , innen = 1; end

function innen = inangle(P,a) % if innen = k ==> zero lies in kth angular wedge closest to edge k
sP2 = size(P,2); E = P(1:2,2:end) - P(1:2,1:end-1); c = inout(P,a); d = []; innen = [];
for j = 1:sP2-1
  Ej = E(:,j); nj = [Ej(2);-Ej(1)]; nj = nj/norm(nj); % outward edge normal
  dist = P(1:2,j)'*nj; al = -P(1:2,j)'*Ej/(Ej'*Ej);   % signed projection distances
  if al >= 0 && al <= 1, d = [d,[dist;j]]; end, end
if isempty(d) == 0,
  j = find(d(1,:) < 0); dmi = d(1,j); [~, k] = min(-c*dmi); innen = d(2,j(k)); end % closest edge

function En = edgenormals(P)     % compute the outward edge normals from points in P
E = P(1:2,2:end)-P(1:2,1:end-1); Ec = E(1,:)+1i*E(2,:); Eca = abs(Ec); % edges
En = [0 1;-1 0]*[E(1,:)./Eca; E(2,:)./Eca];          % normalized inward normals

function [Rout Pout]= eigval(A,Atest,n,opts,kryln,two)  % evaluate extreme eigenvalues
if nargin <= 5, two = 0; end
if n >= kryln,                                        % Use Krylov method for n >= kryln
  [V,~] = eigs(Atest,1,'sr',opts); P = V'*A*V; Rout = [real(P);imag(P)]; % smallest real ev
  if two == 2, % opts.tol = 10^-7; % low accuracy only for the start-up phase largest evalue
    [V,~] = eigs(Atest,1,'lr',opts); Q = V'*A*V; Pout = [real(Q);imag(Q)]; end
else                                                  % otherwise use QR algorithm :
  [V,~] = eig(Atest); P = V(:,1)'*A*V(:,1); Rout = [real(P);imag(P)];  % min ev
  if two == 2                                         % only for the start-up phase
    Q = V(:,n)'*A*V(:,n); Pout = [real(Q);imag(Q)];   % max ev
  else Pout = []; end, end

function wber3(A,L,opts,gr,kryln)  % plot field of values boundary :
[~, n] = size(A); PP = NaN * ones(2*(L+1),1);         % definitions
H = (A+A')/2; K = (A-A')/(2i);                        % hermitean and skewherm parts
th = 0:pi/L:pi; ko = cos(th); si = sin(th);           % set up angular values
for k = 1:L+1,                                        % rotated evalue computations
  Ath = ko(k) * H + si(k) * K;
  if n < kryln                                        % for small n < kryln use QR algorithm
    [V ~] = eig(Ath);
    Vt = V(:,n); PP(k) = Vt'*A*Vt;                    % desired evalues/evectors
    Vt = V(:,1); PP(L+1+k) = Vt'*A*Vt;                % (use max and min evalue/vector)
  else                                                % for large n use Krylov method
    [V ~] = eigs(Ath,1,'Lr',opts); PP(k) = V'*A*V;
    [V ~] = eigs(Ath,1,'Sr',opts); PP(L+1+k) = V'*A*V; end, end
if gr == 1, plot(real(PP),imag(PP),'k'); hold on, grid on, % 1:1 aspect ratio for plot :
  plot(0,0,'or'), set(gca,'PlotBoxAspectRatio',[1,1,1]);  set(gca,'DataAspectRatio',[1,1,1]);
  title(['Numerical range of a ',num2str(n),' by ',num2str(n),' matrix A'],'FontSize',14),
  xlabel('real  axis', 'FontSize',12), ylabel('imaginary  axis', 'FontSize',12), end,
