function [w] = numranges(A, clr)
% numrange routine, modified to support eigs, and linear operator A.
%   
%            An Effective Algorithm for Calculating the Numerical Range
%                        Carl C. Cowen and Elad Harel
%                             Purdue University
%
%  This script finds the numerical range of an n x n matrix by 
%   finding the real and imaginary parts of rotates of the matrix
%   and finding the associated boundary point of that rotate by 
%   finding the largest eigenvalue of the real part and using the
%   corresponding eigenvector's contribution to the numerical range.
%   Multiplicity of the largest eigenvalue, as occurs in a normal 
%   matrix, is handled by plotting the end points of the corresponding
%   segment in the boundary of the numerical range.
%


if nargin<2, clr = 'y'; end
	
%nm=ceil(norm(A, 1));

%th=[0:.01:6.29];
%w=zeros(1,630);

NN = 500;
w=zeros(1,NN);
th = linspace(0, 2*pi, NN);


for j=1:NN
%    Ath=(exp(i*(-th(j))))*A;
%    Hth=(Ath+Ath')/2;
%    [r e]=eig(Hth); 
    [r e]=eig_eval(A, th(j));  %DING

    e=real(diag(e));
    m=max(e);
	s=find(e==m);
 if size(s,1)==1 

        if ~isstruct(A)
	        w(j)=r(:,s)'*A*r(:,s);
        else
	        w(j)=r(:,s)'*A.matmat(r(:,s));
        end
  %
  %  This is the point of the numerical range contributed by 
  %   v_t=r(:,s) when the eigenspace of Hth (H_t) is one dimensional.
  %
	else
%	  Kth=i*(Hth-Ath);
%     pKp=r(:,s)'*Kth*r(:,s);

        rt = r(:,s);
        rtt = exp(i*(-th(j)));
      pKp=.5 * i*rt'*( rtt'*(A'*rt) - rtt*(A*rt)); %DING

  %
  %  The matrix Q described above is  r(:,s)
  %
		[rr ee]=eig(pKp);
		ee=real(diag(ee));
		mm=min(ee);
		sm=find(ee==mm);
        if ~isstruct(A)
		    temp=rr(:,sm(:,1))'*r(:,s)'*A*r(:,s)*rr(:,sm(:,1));
        else
		    temp=rr(:,sm(:,1))'*r(:,s)'*A.matmat(r(:,s))*rr(:,sm(:,1));
        end
		w(j)=temp(1,1);
  %
  %  This is the point of the numerical range contributed by
  %    v_t^- = r(:,s)*rr(:,sm(:,1))
  %
		mM=max(ee);
		sM=find(ee==mM);
        if ~isstruct(A)
		    temp=rr(:,sM(:,1))'*r(:,s)'*A*r(:,s)*rr(:,sM(:,1));
        else
		    temp=rr(:,sM(:,1))'*r(:,s)'*A.matmat(r(:,s))*rr(:,sM(:,1));
        end
		w(j)=temp(1,1);
  %
  %  This is the point of the numerical range contributed by v_t^+
  %
 end
end
fill(real(w),imag(w),clr)
%axis([-nm,nm,-nm,nm])
%axis('equal')

return

% ------------------- AUX FUN -----------------
function [r e]=eig_eval(A, theta); %DING
%    Ath=(exp(i*(-th(j))))*A;
%    Hth=(Ath+Ath')/2;
%    [r e]=eig(Hth); 
if ~isstruct(A)
    Hfun = @(x) 0.5*( exp(-1i*theta)* (A*x) + exp(1i*theta) * (A'*x) );
    n = length(A);
else
    Hfun = @(x) 0.5*( exp(-1i*theta)* (A.matvec(x)) + exp(1i*theta) * (A.rmatvec(x)) );
    n = A.n; 
end
opts.isreal = false;
opts.issym = true;
opts.maxit = 2000;
[r, e] = eigs(Hfun, n , 30, 'lr', opts);

return
